/* Nessus Attack Scripting Language
 *
 * Copyright (C) 1999 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include "../include/config.h"
#include <includes.h>
#include <nasl_raw.h>
#include <pcap.h>
#include "nasl_memory.h"

#ifdef _CYGWIN_
extern unsigned short * SChar2WChar(char *);
extern char * WChar2SChar(unsigned short *);
#endif

extern int islocalhost(struct in_addr *);
int init_ip_pcap(harglst*, struct in_addr src, struct in_addr dst,
		       char *);




/*
 * Set up the pcap filter, and select the correct interface.
 *
 * The filter will be changed only if this is necessary
 * 
 */
 
int
init_ip_pcap(globals, src, dst, filter)
 harglst * globals;
 struct in_addr src, dst;
 char * filter;
{
 int ret;
 char * interface = NULL;
 char * a_dst, *a_src;
 char errbuf[PCAP_ERRBUF_SIZE];
 
 a_src = nasl_strdup(globals, inet_ntoa(src));
 a_dst = nasl_strdup(globals, inet_ntoa(dst));
 if((!filter) || (filter[0]=='\0') || (filter[0]=='0'))
 {
  filter = nasl_malloc(globals, 1024);
  if(!islocalhost(&dst))
  	sprintf(filter, "ip and (src host %s and dst host %s)",
 		  a_src, a_dst);
		  
 }
 else {
 	if(!islocalhost(&dst))filter = nasl_strdup(globals, filter);
	else filter = nasl_malloc(globals, 1);
 	}
 
 nasl_free(globals, a_dst);
 nasl_free(globals, a_src);

 if((interface = routethrough(&dst, &src))||
    (interface = pcap_lookupdev(errbuf)))   
    ret = bpf_open_live(interface, filter);
 return(ret);
}



struct ip * 
recv_ip_packet(globals, bpf, timeout)
 harglst * globals;
 int bpf;
 int timeout;
{
  struct pcap_pkthdr head;
  int dl_len;
  char * packet = NULL;
  char * ret = NULL;
  struct timeval past, now;
  struct timezone tz;
 
  if(bpf < 0)
   return NULL;
   
  dl_len =  get_datalink_size(bpf_datalink(bpf));
  bzero(&past, sizeof(past));
  bzero(&now, sizeof(now));
  gettimeofday(&past, &tz);
  for(;;)
  {
   packet = (char*)bpf_next(bpf, &head);
   if(packet != NULL)
     break;
   gettimeofday(&now, &tz);
   if((now.tv_sec - past.tv_sec)>=timeout)break;
  }
  
  if(packet)
  {
   struct ip * ip;
   ip = (struct ip *)(packet + dl_len);
#ifdef BSD_BYTE_ORDERING
   ip->ip_len = ntohs(ip->ip_len);
   ip->ip_off = ntohs(ip->ip_off);
#endif   
   ip->ip_id = ntohs(ip->ip_id);
   ret = nasl_malloc(globals, UNFIX(ip->ip_len));
   memcpy(ret, ip, UNFIX(ip->ip_len));
#if 0   
   dump_packet(ret, 40);
#endif   
  }
 return((struct ip*)ret);
}
